@@ -108,7 +108,7 @@ char *__acpi_map_table(unsigned long phys_addr, unsigned long size)
 	if (!phys_addr || !size)
 		return NULL;
 
-	if (phys_addr < (end_pfn_map << PAGE_SHIFT))
+	if (phys_addr+size <= (end_pfn_map << PAGE_SHIFT) + PAGE_SIZE)
 		return __va(phys_addr);
 
 	return NULL;
@@ -215,7 +215,7 @@ static int __init acpi_parse_madt(unsigned long phys_addr, unsigned long size)
 {
 	struct acpi_table_madt *madt = NULL;
 
-	if (!phys_addr || !size)
+	if (!phys_addr || !size || !cpu_has_apic)
 		return -EINVAL;
 
 	madt = (struct acpi_table_madt *)__acpi_map_table(phys_addr, size);
@@ -621,9 +621,9 @@ extern u32 pmtmr_ioport;
 
 static int __init acpi_parse_fadt(unsigned long phys, unsigned long size)
 {
-	struct fadt_descriptor_rev2 *fadt = NULL;
+	struct fadt_descriptor *fadt = NULL;
 
-	fadt = (struct fadt_descriptor_rev2 *)__acpi_map_table(phys, size);
+	fadt = (struct fadt_descriptor *)__acpi_map_table(phys, size);
 	if (!fadt) {
 		printk(KERN_WARNING PREFIX "Unable to map FADT\n");
 		return 0;
@@ -668,10 +668,10 @@ unsigned long __init acpi_find_rsdp(void)
 	unsigned long rsdp_phys = 0;
 
 	if (efi_enabled) {
-		if (efi.acpi20)
-			return __pa(efi.acpi20);
-		else if (efi.acpi)
-			return __pa(efi.acpi);
+		if (efi.acpi20 != EFI_INVALID_TABLE_ADDR)
+			return efi.acpi20;
+		else if (efi.acpi != EFI_INVALID_TABLE_ADDR)
+			return efi.acpi;
 	}
 	/*
 	 * Scan memory looking for the RSDP signature. First search EBDA (low
@@ -693,6 +693,9 @@ static int __init acpi_parse_madt_lapic_entries(void)
 {
 	int count;
 
+	if (!cpu_has_apic)
+		return -ENODEV;
+
 	/* 
 	 * Note that the LAPIC address is obtained from the MADT (32-bit value)
 	 * and (optionally) overriden by a LAPIC_ADDR_OVR entry (64-bit value).
@@ -750,6 +753,11 @@ static int __init acpi_parse_madt_ioapic_entries(void)
 	if (acpi_disabled || acpi_noirq) {
 		return -ENODEV;
 	}
+	if (!cpu_has_apic)
+		return -ENODEV;
+
+	if (!cpu_has_apic)
+		return -ENODEV;
 
 	/*
 	 * if "noapic" boot option, don't look for IO-APICs
@@ -922,7 +930,15 @@ static struct dmi_system_id __initdata acpi_dmi_table[] = {
 		     DMI_MATCH(DMI_BOARD_NAME, "2629H1G"),
 		     },
 	 },
-
+	 {
+	  .callback = dmi_disable_acpi,
+	  .ident = "Phoenix Technologies",
+	  .matches = {
+	  	      DMI_MATCH(DMI_BIOS_VENDOR,"Phoenix Technologies, LTD"),
+		      DMI_MATCH(DMI_BIOS_VERSION, "6.00 PG"),
+		      DMI_MATCH(DMI_BIOS_DATE, "11/05/2003"),
+	  	     },
+	 },
 	/*
 	 * Boxes that need acpi=ht
 	 */
@@ -1055,6 +1071,14 @@ static struct dmi_system_id __initdata acpi_dmi_table[] = {
 		     DMI_MATCH(DMI_BIOS_VERSION, "0306"),
 		    }
 	},
+	{
+	 .callback = disable_acpi_irq,
+	 .ident = "Averatec 3200",
+	 .matches = {
+		     DMI_MATCH(DMI_BOARD_VENDOR, "AVERATEC"),
+		     DMI_MATCH(DMI_BOARD_NAME, "3200"),
+		    }
+	},
 
 	/*
 	 * Boxes that need ACPI PCI IRQ routing and PCI scan disabled
@@ -1097,6 +1121,14 @@ static struct dmi_system_id __initdata acpi_dmi_table[] = {
 		     DMI_MATCH(DMI_BIOS_DATE, "06/23/2005"),
 		    },
 	},
+	{
+	 .callback = disable_acpi_pci,
+	 .ident = "HP xw9300",
+	 .matches = {
+		    DMI_MATCH(DMI_SYS_VENDOR, "Hewlett-Packard"),
+		    DMI_MATCH(DMI_PRODUCT_NAME, "HP xw9300 Workstation"),
+		    },
+	},
 	{}
 };
 
