@@ -1865,7 +1865,7 @@ static inline void check_timer(void)
 			return;
 		}
 		clear_IO_APIC_pin(apic1, pin1);
-		apic_printk(APIC_QUIET,KERN_ERR "..MP-BIOS bug: 8254 timer not "
+		apic_printk(APIC_QUIET,KERN_WARNING "..MP-BIOS bug: 8254 timer not "
 				"connected to IO-APIC\n");
 	}
 
