@@ -340,7 +340,7 @@ static void __init find_early_table_space(unsigned long end)
 	table_start >>= PAGE_SHIFT;
 	table_end = table_start;
 
-	early_printk("kernel direct mapping tables up to %lx @ %lx-%lx\n",
+	Dprintk("kernel direct mapping tables up to %lx @ %lx-%lx\n",
 		end, table_start << PAGE_SHIFT, table_end << PAGE_SHIFT);
 }
 
