@@ -279,7 +279,7 @@ static int acpi_pcc_retrieve_biosdata(struct acpi_hotkey *hotkey, u32* sinf)
 	sinf[hkey->package.count] = -1;
 
 end:
-	acpi_os_free(buffer.pointer);
+	kfree(buffer.pointer);
 	return_VALUE(status == AE_OK);
 }
 
