@@ -65,6 +65,21 @@
 
 #include <acpi/acpi_drivers.h>
 
+/* Some compatibility for isa legacy interface */
+#ifndef isa_readb
+
+#define isa_readb(a) readb(__ISA_IO_base + (a))
+#define isa_readw(a) readw(__ISA_IO_base + (a))
+#define isa_readl(a) readl(__ISA_IO_base + (a))
+#define isa_writeb(b,a) writeb(b,__ISA_IO_base + (a))
+#define isa_writew(w,a) writew(w,__ISA_IO_base + (a))
+#define isa_writel(l,a) writel(l,__ISA_IO_base + (a))
+#define isa_memset_io(a,b,c)            memset_io(__ISA_IO_base + (a),(b),(c))
+#define isa_memcpy_fromio(a,b,c)        memcpy_fromio((a),__ISA_IO_base + (b),(c))
+#define isa_memcpy_toio(a,b,c)          memcpy_toio(__ISA_IO_base + (a),(b),(c))
+
+#endif
+
 MODULE_AUTHOR("John Belmonte");
 MODULE_DESCRIPTION("Toshiba Laptop ACPI Extras Driver");
 MODULE_LICENSE("GPL");
@@ -236,8 +251,8 @@ static int key_event_valid;
 static int hotkeys_over_acpi = 1;
 static int hotkeys_check_per_sec = 2;
 
-MODULE_PARM(hotkeys_over_acpi, "i");
-MODULE_PARM(hotkeys_check_per_sec, "i");
+module_param(hotkeys_over_acpi, uint, 0400);
+module_param(hotkeys_check_per_sec, uint, 0400);
 
 
 typedef struct _ProcItem {
@@ -816,9 +831,9 @@ static int __init init_threaded_acpi(void)
 {
         acpi_status result = AE_OK;
         result = acpi_bus_register_driver(&acpi_threaded_toshkeys);
-        if( result != 1 )
+        if( result < 0 )
                 printk(MY_ERR "Registration of toshkeys acpi device failed\n");
-        return result != 1;
+        return result;
 }
 
 static void kill_threaded_acpi(void)
@@ -879,7 +894,7 @@ static int __init toshiba_acpi_init(void)
 			hotkeys_check_per_sec = 10;
 		printk(MY_INFO "ktoshkeyd will check %d time%s per second\n",
 			hotkeys_check_per_sec, hotkeys_check_per_sec==1?"":"s");
-		if (!init_threaded_acpi()) {
+		if (init_threaded_acpi() >= 0) {
 			init_MUTEX_LOCKED(&thread_sem);
 			kernel_thread(toshiba_acpi_thread, NULL, CLONE_KERNEL);
 			down(&thread_sem);
