@@ -187,10 +187,10 @@ int poll = 0;
 
 #ifdef MODULE
 MODULE_AUTHOR("Andreas Eversberg");
-MODULE_PARM(debug, "1i");
-MODULE_PARM(options, "1i");
+module_param(debug, uint, 0400);
+module_param(options, uint, 0400);
 #ifndef AUTOJITTER
-MODULE_PARM(poll, "1i");
+module_param(poll, uint, 0400);
 #endif
 #ifdef MODULE_LICENSE
 MODULE_LICENSE("GPL");
