@@ -1025,11 +1025,11 @@ MODULE_AUTHOR("Karsten Keil");
 #ifdef MODULE_LICENSE
 MODULE_LICENSE("GPL");
 #endif
-MODULE_PARM(debug, "1i");
-MODULE_PARM(led, MODULE_PARM_T);
-MODULE_PARM(pots, MODULE_PARM_T);
-MODULE_PARM(protocol, MODULE_PARM_T);
-MODULE_PARM(layermask, MODULE_PARM_T);
+module_param(debug, uint, 0400);
+module_param_array(protocol, int, NULL, 0400);
+module_param_array(layermask, int, NULL, 0400);
+module_param_array(pots, int, NULL, 0400);
+module_param_array(led, int, NULL, 0400);
 #endif
 
 /******************************/
@@ -1221,7 +1221,7 @@ w6692_l1hwD(mISDNif_t *hif, struct sk_buff *skb)
 	return(ret);
 }
 
-int __init 
+int __devinit
 setup_w6692(w6692pci *card)
 {
 	u_int	val;
