@@ -82,8 +82,9 @@ MODULE_SUPPORTED_DEVICE("em8300");
 
 MODULE_LICENSE("GPL");
 
-static unsigned int use_bt865[EM8300_MAX]={};
-MODULE_PARM(use_bt865, "1-" __MODULE_STRING(EM8300_MAX) "i");
+static unsigned int use_bt865[EM8300_MAX];
+
+module_param_array(use_bt865, uint, NULL, 0400);
 MODULE_PARM_DESC(use_bt865, "Set this to 1 if you have a bt865. It changes some internal register values. Defaults to 0.");
 
 /*
@@ -94,7 +95,7 @@ int dicom_other_pal = 1;
 #else
 int dicom_other_pal = 0;
 #endif
-MODULE_PARM(dicom_other_pal, "i");
+module_param(dicom_other_pal, uint, 0400);
 MODULE_PARM_DESC(dicom_other_pal, "If this is set, then some internal register values are swapped for PAL and NTSC. Defaults to 1.");
 
 #ifdef CONFIG_EM8300_DICOMFIX
@@ -102,7 +103,7 @@ int dicom_fix = 1;
 #else
 int dicom_fix = 0;
 #endif
-MODULE_PARM(dicom_fix, "i");
+module_param(dicom_fix, uint, 0400);
 MODULE_PARM_DESC(dicom_fix, "If this is set then some internal register values are changed. Fixes green screen problems for some. Defaults to 1.");
 
 #ifdef CONFIG_EM8300_DICOMCTRL
@@ -110,7 +111,7 @@ int dicom_control = 1;
 #else
 int dicom_control = 0;
 #endif
-MODULE_PARM(dicom_control, "i");
+module_param(dicom_control, uint, 0400);
 MODULE_PARM_DESC(dicom_control, "If this is set then some internal register values are changed. Fixes green screen problems for some. Defaults to 1.");
 
 #ifdef CONFIG_EM8300_UCODETIMEOUT
@@ -118,7 +119,7 @@ int bt865_ucode_timeout = 1;
 #else
 int bt865_ucode_timeout = 0;
 #endif
-MODULE_PARM(bt865_ucode_timeout, "i");
+module_param(bt865_ucode_timeout, uint, 0400);
 MODULE_PARM_DESC(bt865_ucode_timeout, "Set this to 1 if you have a bt865 and get timeouts when uploading the microcode. Defaults to 0.");
 
 #ifdef CONFIG_EM8300_LOOPBACK
@@ -126,7 +127,7 @@ int activate_loopback = 1;
 #else
 int activate_loopback = 0;
 #endif
-MODULE_PARM(activate_loopback, "i");
+module_param(activate_loopback, uint, 0400);
 MODULE_PARM_DESC(activate_loopback, "If you lose video after loading the modules or uploading the microcode set this to 1. Defaults to 0.");
 
 static int em8300_cards = 0;
