@@ -37,7 +37,7 @@
 
 unsigned long flagg;
 ULONG	debug = RT_DEBUG_OFF;
-MODULE_PARM(debug, "i");
+module_param(debug, uint, 0400);
 MODULE_PARM_DESC(debug, "Enable level: accepted values: 1 to switch debug on, 0 to switch debug off.");
 
 #define KERNEL_20424	0x20418
@@ -92,7 +92,6 @@ static int usb_rtusb_probe (struct usb_interface *intf,
 static void usb_rtusb_disconnect(struct usb_interface *intf);
 
 struct usb_driver rtusb_driver = {
-	.owner =	THIS_MODULE,
 	.name="rtusb",
 	.probe=usb_rtusb_probe,
 	.disconnect=usb_rtusb_disconnect,
