@@ -16,7 +16,7 @@
  * This Copyright notice must be kept intact and distributed with all sources.
  */
 /*
- *  $Id: copyup.c,v 1.61 2006/01/22 19:46:50 jsipek Exp $
+ *  $Id: copyup.c,v 1.65 2006/02/08 21:33:40 dquigley Exp $
  */
 
 #include "unionfs.h"
@@ -64,7 +64,7 @@ static int copyup_xattrs(struct dentry *old_hidden_dentry,
 	name_list_orig = name_list;
 	while (*name_list) {
 		ssize_t size;
-		down(&old_hidden_dentry->d_inode->i_sem);
+		mutex_lock(&old_hidden_dentry->d_inode->i_mutex);
 		err = security_inode_getxattr(old_hidden_dentry, name_list);
 		if (err)
 			size = err;
@@ -73,7 +73,7 @@ static int copyup_xattrs(struct dentry *old_hidden_dentry,
 			    old_hidden_dentry->d_inode->i_op->
 			    getxattr(old_hidden_dentry, name_list, attr_value,
 				     XATTR_SIZE_MAX);
-		up(&old_hidden_dentry->d_inode->i_sem);
+		mutex_unlock(&old_hidden_dentry->d_inode->i_mutex);
 		if (size < 0) {
 			err = size;
 			goto out;
@@ -84,7 +84,7 @@ static int copyup_xattrs(struct dentry *old_hidden_dentry,
 			goto out;
 		}
 
-		down(&new_hidden_dentry->d_inode->i_sem);
+		mutex_lock(&old_hidden_dentry->d_inode->i_mutex);
 
 		err =
 		    security_inode_setxattr(old_hidden_dentry, name_list,
@@ -101,7 +101,7 @@ static int copyup_xattrs(struct dentry *old_hidden_dentry,
 							     attr_value, size,
 							     0);
 		}
-		up(&new_hidden_dentry->d_inode->i_sem);
+		mutex_unlock(&old_hidden_dentry->d_inode->i_mutex);
 
 		if (err < 0)
 			goto out;
@@ -135,19 +135,7 @@ static int copyup_permissions(struct super_block *sb,
 	newattrs.ia_ctime = old_hidden_dentry->d_inode->i_ctime;
 	newattrs.ia_valid = ATTR_CTIME | ATTR_ATIME | ATTR_MTIME |
 	    ATTR_ATIME_SET | ATTR_MTIME_SET;
-	if (IS_SET(sb, COPYUP_FS_MOUNTER)) {
-		/* f/s mounter */
-		newattrs.ia_mode = stopd(sb)->copyupmode;
-		newattrs.ia_valid |= ATTR_FORCE | ATTR_MODE;
-		if (stopd(sb)->copyupuid != -1) {
-			newattrs.ia_uid = stopd(sb)->copyupuid;
-			newattrs.ia_valid |= ATTR_UID;
-		}
-		if (stopd(sb)->copyupgid != -1) {
-			newattrs.ia_gid = stopd(sb)->copyupgid;
-			newattrs.ia_valid |= ATTR_GID;
-		}
-	} else if (IS_SET(sb, COPYUP_CURRENT_USER)) {
+	if (IS_SET(sb, COPYUP_CURRENT_USER)) {
 		/* current file permission */
 		newattrs.ia_mode = ~current->fs->umask & S_IRWXUGO;
 		newattrs.ia_valid |= ATTR_FORCE | ATTR_MODE;
@@ -298,14 +286,14 @@ int copyup_named_dentry(struct inode *dir, struct dentry *dentry,
 		set_dbend(dentry, old_bend);
 		goto out_dir;
 	}
-
+#ifdef UNIONFS_IMAP
 	if (stopd(sb)->usi_persistent) {
 		err = write_uin(dentry->d_sb, dentry->d_inode->i_ino,
 				new_bindex, new_hidden_dentry->d_inode->i_ino);
 		if (err)
 			goto out_dir;
 	}
-
+#endif
 	/* We actually copyup the file here. */
 	if (S_ISREG(old_hidden_dentry->d_inode->i_mode)) {
 		mntget(stohiddenmnt_index(sb, old_bindex));
@@ -353,6 +341,7 @@ int copyup_named_dentry(struct inode *dir, struct dentry *dentry,
 		input_file->f_pos = 0;
 		output_file->f_pos = 0;
 
+		err = 0;	// reset error just in case
 		set_fs(KERNEL_DS);
 		do {
 			if (len >= PAGE_SIZE)
@@ -377,12 +366,22 @@ int copyup_named_dentry(struct inode *dir, struct dentry *dentry,
 						     read_bytes,
 						     &output_file->f_pos);
 			if (write_bytes < 0 || (write_bytes < read_bytes)) {
-				err = -EIO;
+				err = write_bytes;
 				break;
 			}
 		} while ((read_bytes > 0) && (len > 0));
 		set_fs(old_fs);
 		KFREE(buf);
+
+		if (err) {
+			/* copyup failed, because we ran out of space or quota,
+			 * or something else happened so let's unlink; we don't
+			 * really care about the return value of vfs_unlink */
+			vfs_unlink(new_hidden_parent_dentry->d_inode,
+				   new_hidden_dentry);
+
+			goto out_dir;
+		}
 	}
 
 	/* Set permissions. */
@@ -400,6 +399,7 @@ int copyup_named_dentry(struct inode *dir, struct dentry *dentry,
       out_dir:
 	if (new_hidden_parent_dentry)
 		unlock_dir(new_hidden_parent_dentry);
+
       out:
 	if (input_file && !IS_ERR(input_file)) {
 		fput(input_file);
@@ -418,9 +418,10 @@ int copyup_named_dentry(struct inode *dir, struct dentry *dentry,
 		if (copyup_file && !err) {
 			*copyup_file = output_file;
 		} else {
-			if (!err) {
+			/* close the file if there was no error, or if we ran
+			 * out of space in which case we unlinked the file */
+			if (!IS_ERR(output_file))
 				fput(output_file);
-			}
 			branchput(sb, got_branch_output);
 		}
 	}
@@ -519,8 +520,9 @@ struct dentry *create_parents_named(struct inode *dir, struct dentry *dentry,
 	struct dentry **path = NULL;
 	struct dentry **tmp_path;
 	struct super_block *sb;
+#ifdef UNIONFS_IMAP
 	int persistent;
-
+#endif
 	print_entry_location();
 
 	verify_locked(dentry);
@@ -590,8 +592,9 @@ struct dentry *create_parents_named(struct inode *dir, struct dentry *dentry,
 	count--;
 
 	sb = dentry->d_sb;
+#ifdef UNIONFS_IMAP
 	persistent = stopd(sb)->usi_persistent;
-
+#endif
 	/* This is basically while(child_dentry != dentry).  This loop is
 	 * horrible to follow and should be replaced with cleaner code. */
 	while (1) {
@@ -693,6 +696,7 @@ struct dentry *create_parents_named(struct inode *dir, struct dentry *dentry,
 				hidden_dentry = ERR_PTR(err);
 				goto out;
 			}
+#ifdef UNIONFS_IMAP
 			if (persistent) {
 				err = write_uin
 				    (sb, child_dentry->d_inode->i_ino,
@@ -703,6 +707,7 @@ struct dentry *create_parents_named(struct inode *dir, struct dentry *dentry,
 					goto out;
 				}
 			}
+#endif
 			set_itohi_index(child_dentry->d_inode, bindex,
 					IGRAB(hidden_dentry->d_inode));
 			if (ibstart(child_dentry->d_inode) > bindex)
