@@ -16,7 +16,7 @@
  * This Copyright notice must be kept intact and distributed with all sources.
  */
 /*
- *  $Id: inode.c,v 1.260 2006/01/21 02:58:11 jsipek Exp $
+ *  $Id: inode.c,v 1.263.2.1 2006/04/11 19:37:35 jsipek Exp $
  */
 
 #include "unionfs.h"
@@ -81,15 +81,28 @@ static int unionfs_create(struct inode *parent, struct dentry *dentry,
 		struct dentry *hidden_dir_dentry;
 		struct iattr newattrs;
 
-		down(&whiteout_dentry->d_inode->i_sem);
-		newattrs.ia_valid = ATTR_CTIME | ATTR_MODE;
-		newattrs.ia_mode = mode;
+		mutex_lock(&whiteout_dentry->d_inode->i_mutex);
+		newattrs.ia_valid = ATTR_CTIME | ATTR_MODE | ATTR_ATIME
+		    | ATTR_MTIME | ATTR_UID | ATTR_GID | ATTR_FORCE
+		    | ATTR_KILL_SUID | ATTR_KILL_SGID;
+
+		newattrs.ia_mode = mode & ~current->fs->umask;
+		newattrs.ia_uid = current->fsuid;
+		newattrs.ia_gid = current->fsgid;
+
 		if (whiteout_dentry->d_inode->i_size != 0) {
 			newattrs.ia_valid |= ATTR_SIZE;
 			newattrs.ia_size = 0;
 		}
+
 		err = notify_change(whiteout_dentry, &newattrs);
-		up(&whiteout_dentry->d_inode->i_sem);
+
+		mutex_unlock(&whiteout_dentry->d_inode->i_mutex);
+
+		if (err)
+			printk(KERN_WARNING
+			       "unionfs: %s:%d: notify_change failed: %d, ignoring..\n",
+			       __FILE__, __LINE__, err);
 
 		new_hidden_dentry = dtohd(dentry);
 		DGET(new_hidden_dentry);
@@ -581,10 +594,10 @@ static int unionfs_mkdir(struct inode *parent, struct dentry *dentry, int mode)
 				err = PTR_ERR(whiteout_dentry);
 				goto out;
 			}
-			down(&hidden_dentry->d_inode->i_sem);
+			mutex_lock(&hidden_dentry->d_inode->i_mutex);
 			err = vfs_create(hidden_dentry->d_inode,
 					 whiteout_dentry, 0600, NULL);
-			up(&hidden_dentry->d_inode->i_sem);
+			mutex_unlock(&hidden_dentry->d_inode->i_mutex);
 			DPUT(whiteout_dentry);
 
 			if (err) {
@@ -838,6 +851,7 @@ static int inode_permission(struct inode *inode, int mask, struct nameidata *nd,
 		retval = inode->i_op->permission(inode, submask, nd);
 		if ((retval == -EACCES) && (submask & MAY_WRITE) &&
 		    (!strcmp("nfs", (inode)->i_sb->s_type->name)) &&
+		    (nd) && (nd->mnt) && (nd->mnt->mnt_sb) &&
 		    (branchperms(nd->mnt->mnt_sb, bindex) & MAY_NFSRO)) {
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,10)
 			retval = vfs_permission(inode, submask);
@@ -864,7 +878,8 @@ static int unionfs_permission(struct inode *inode, int mask,
 	struct inode *hidden_inode = NULL;
 	int err = 0;
 	int bindex, bstart, bend;
-	int is_file = 0;
+	const int is_file = !S_ISDIR(inode->i_mode);
+	const int write_mask = (mask & MAY_WRITE) && !(mask & MAY_READ);
 
 	print_entry_location();
 
@@ -873,10 +888,6 @@ static int unionfs_permission(struct inode *inode, int mask,
 
 	fist_print_inode("IN unionfs_permission", inode);
 
-	/* set if check is for file */
-	if (!S_ISDIR(inode->i_mode))
-		is_file = 1;
-
 	for (bindex = bstart; bindex <= bend; bindex++) {
 		hidden_inode = itohi_index(inode, bindex);
 		if (!hidden_inode)
@@ -886,7 +897,7 @@ static int unionfs_permission(struct inode *inode, int mask,
 		 * we dont have to check for files, if we are checking for
 		 * directories.
 		 */
-		if (!S_ISDIR(hidden_inode->i_mode) && (!is_file))
+		if (!is_file && !S_ISDIR(hidden_inode->i_mode))
 			continue;
 		/* We use our own special version of permission, such that 
 		 * only the first branch returns -EROFS. */
@@ -896,12 +907,11 @@ static int unionfs_permission(struct inode *inode, int mask,
 		if (err)
 			goto out;
 		/* only the leftmost file matters. */
-		if (is_file) {
-			if (mask & MAY_WRITE) {
+		if (is_file || write_mask) {
+			if (is_file && write_mask) {
 				err = get_write_access(hidden_inode);
-				if (err)
-					goto out;
-				put_write_access(hidden_inode);
+				if (!err)
+					put_write_access(hidden_inode);
 			}
 			break;
 		}
